﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using Microsoft.SqlServer.Dts.Runtime;
using System.ServiceModel;
using System.Text;
using SqlBI.DtExec.Contracts;
using SqlBI.DtExec.Classes;

namespace SqlBI.DtExec.Services {
    /// <summary>
    /// 
    ///     This class implements the WCF service
    /// 
    /// </summary>
    public class ServiceSqlBIDtExec : IServiceSqlBIDtExec {
        /// <summary>
        /// 
        ///     Contains the callback delegate
        /// 
        /// </summary>
        ILogCallback callback;

        /// <summary>
        /// 
        ///     Sends a message back to the client
        /// 
        /// </summary>
        /// <param name="message"></param>
        public void SendBackMessage (LogType type, string message) {
            lock (this) {
                LogItem logItem = new LogItem ();
                logItem.TimeStamp = DateTime.Now;
                logItem.Message = message;
                logItem.LogType = type;
                callback.OnLogMessage (logItem);
            }
        }

        /// <summary>
        /// 
        ///     Runs the package
        /// 
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public string RunPackage (PackageInformation package) {
            callback = OperationContext.Current.GetCallbackChannel<ILogCallback> ();

            try {

                SendBackMessage (LogType.Log, "Loading package");

                Microsoft.SqlServer.Dts.Runtime.Application integrationServices = new Application ();

                //
                //  If it has been specified, we set up the password to decrypt
                //  the package upon loading
                //
                if (package.Password != "") {
                    integrationServices.PackagePassword = package.Password;
                }
                Package packageToRun = integrationServices.LoadPackage (package.Path, null);

                SendBackMessage (LogType.Log, "Package configurations: ");
                foreach (Configuration c in packageToRun.Configurations) {

                    SendBackMessage (LogType.Log, "Configuration: " + c.Name);
                    SendBackMessage (LogType.Log, "    points to: " + c.ConfigurationString);

                    foreach (PackageConfiguration config in package.Configurations) {
                        if (config.Name == c.Name) {
                            DtsProperty propertyToSet = null;
                            var parentItem = packageToRun.GetObjectFromPackagePath (c.PackagePath, out propertyToSet);
                            propertyToSet.SetValue (parentItem, config.Value);
                            SendBackMessage (LogType.Log, "Value set to   : " + config.Value);
                            break;
                        }
                    }
                }
                SendBackMessage (LogType.Log, "End of Package configurations");

                if (packageToRun.Errors.Count > 0) {
                    SendBackMessage (LogType.Log, String.Format ("There were errors during the loading of '{0}'", package.Name));
                    foreach (var err in packageToRun.Errors) {
                        SendBackMessage (LogType.Log, "Error: " + err.Description);
                    }
                    return "Failure";
                } else {
                    LoggingHandler logger = new LoggingHandler (this, package.Logging);
                    EventHandler eventHandler = new EventHandler (this, package.Events);
                    DTSExecResult packageResult = packageToRun.Execute (null, null, eventHandler, logger, null);
                    return packageResult.ToString ();
                }
            } catch (Exception e) {
                SendBackMessage (LogType.Log, "Error during package loading: " + e.Message);
                return "Failure";
            }
        }
    }
}
