﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using SqlBI.DtExec.Classes;

namespace SqlBI.DtExec.TestApplication {

    [XmlRootAttribute ("Configuration")]
    public class Configuration {
        private List<PackageInformation> packages = new List<PackageInformation> ();
        private EventsOption defaultEvents = new EventsOption ();
        private LoggingOptions defaultLogging = new LoggingOptions ();
        private string defaultEndPointToUse = "";
        private string timeStampFormat = "MM/dd/yyyy HH:mm";

        /// <summary>
        /// 
        ///     The format to use to provide time stamps in the log
        /// 
        /// </summary>
        public string TimeStampFormat {
            get { return timeStampFormat; }
            set { timeStampFormat = value; }
        }

        /// <summary>
        /// 
        ///     The default endpoint to use, can be specified as a default
        ///     value or for each single package
        /// 
        /// </summary>
        public string DefaultEndPointToUse {
            get { return defaultEndPointToUse; }
            set { defaultEndPointToUse = value; }
        }

        /// <summary>
        /// 
        ///     Default event information
        /// 
        /// </summary>
        public EventsOption DefaultEvents {
            get { return defaultEvents; }
            set { defaultEvents = value; }
        }

        /// <summary>
        /// 
        ///     Default logging information
        /// 
        /// </summary>
        public LoggingOptions DefaultLogging {
            get { return defaultLogging; }
            set { defaultLogging = value; }
        }

        /// <summary>
        /// 
        ///     List of the packages that need to be executed
        /// 
        /// </summary>
        public List<PackageInformation> Packages {
            get { return packages; }
            set { packages = value; }
        }

        /// <summary>
        /// 
        ///     Applies the default rules to each package, if a specific rule
        ///     has not been supplied
        /// 
        /// </summary>
        public void ApplyDefaultRules () {
            foreach (PackageInformation package in Packages) {
                //
                //  Let us handle the default values for logging and events,
                //  if nothing has been specified in the package information,
                //  then use the default ones
                //
                if (package.Logging == null) {
                    package.Logging = DefaultLogging;
                }
                if (package.Events == null) {
                    package.Events = DefaultEvents;
                }

                //
                //  Apply the default end point
                //  
                if (package.EndPointToUse == "" || package.EndPointToUse == null) {
                    package.EndPointToUse = DefaultEndPointToUse;
                }
            }
        }
    }
}
